/* compile with "gcc xtest.c -lX11 -o xtest"
 *
 * Demonstrate a problem in the XCopyPlane() routine in XFree 3.1.2
 * Please refer to the following program code for a description of what
 * is going wrong, and how to reproduce / recognize it.
 * You can jump to *prob* to skip the intializations.
  
   Test configurations which reproduce the problem:
   ------------------------------------------------

  - i486DX-33 w/ HOT-409 VLB board
  - 20 MB RAM
  - Linux 1.2.13 ELF (slackware 3.0)
  - Cirrus 5424 ISA graphics card, 1 MB DRAM
  - XFree SVGA servers 3.1.1, 3.1.2, 3.1.2B, 3.1.2D, 3.1.2E

  - P100 w/ intel Zappa board
  - 16 MB RAM
  - Linux 1.2.13 ELF (slackware 3.0)
  - ATI expression board, 1 MB DRAM, CT220 internal Ramdac
  - XFree Mach64 server 3.1.2B

  Test configurations which do *not* reproduce the problem:
  ---------------------------------------------------------

  - The 486 system from above using
    . the XFree SVGA server 2.1.1 or
    . the Accelerated-X 1.2 demo server

  - The Pentium system from above using
    . the XFree Mach64 server 3.1.2E
    . the Accelerated-X 1.2 demo server

  - a Sparc-5 running Solaris 2.4 and the standard X server
  */

#include <X11/Xlib.h>
#include <stdio.h>

void Error(char *err)
{  printf("%s\n",err);
   exit(-10);
}

main()
{  Display *disp;	/* These are obtained as default values from the server */
   Window win;
   Screen *screen;
   int screen_no;
   unsigned long white,black;	/* color values for white and black colors */

   XSetWindowAttributes setattr;/* needed for opening the window */
   unsigned long attrmask;

   XGCValues gcvalues;		/* needed for creating the graphics context */
   unsigned long gcmask;

   Pixmap black_map;		/* 1-bit deep pixmap to demonstrate the problem */
   Pixmap tri_map;		/* another 1-bit pixmap for the triangle clipmask */
   GC monogc;			/* GC for drawing into the pixmaps */
   GC gc;			/* GC for drawing into the window */
   XEvent event;

   XPoint xp[3];		/* used for the definition of the triangle  */

	/* retrieve some basic information from the X server */

   if ( !(disp = XOpenDisplay(NULL)) ) Error("Can't open Display");
   screen_no	= XDefaultScreen(disp);
   screen	= XScreenOfDisplay(disp,screen_no);
   gc		= XDefaultGC(disp,screen_no);
   white	= XWhitePixel(disp,screen_no);
   black	= XBlackPixel(disp,screen_no);

	/* open the window */

   setattr.background_pixel = white;
   setattr.event_mask = ButtonPressMask | ExposureMask;
   attrmask = CWBackPixel | CWEventMask;

   if ( !(win = XCreateWindow(disp, XDefaultRootWindow(disp),
                                    10, 10, 300, 200,
                                    2,
                                    CopyFromParent,
                                    InputOutput,
                                    CopyFromParent,
                                    attrmask,
                                    &setattr)) )
     Error("Can't create Window");

	/* map the window and wait for it to come up */

   XMapWindow(disp,win);
   XMaskEvent(disp,ExposureMask,&event);


#define WIDTH 101
#define HEIGHT 50

	/* ------------------------------------------------------------------- */
	/* *prob* demonstration XCopyPlane() problem begins here ------------- */
	/* ------------------------------------------------------------------- */
	/* When combining XCopyPlane() with a ClipMask, some of the rightmost  */
        /* pixels in the clipmask are dropped / ignored.
	/* ------------------------------------------------------------------- */

   	/* Prepare a XPoint array to define an upside-down triangle,
	 * which can then get rendered by a XFillPolygon() call.
	 * The triangle will later be used as a clipmask to demonstrate the
	 * problem. The shape of a triangle is chosen to demonstrate that
	 * dropped pixels will also occur inside the pixmap, not just on its
	 * outer sides. Therefore, we're not just experiencing a rectangular
	 * clipping effect at the right pixmap border.
	 */

   xp[0].x = 0;   		xp[0].y =  0;	/* define the triangle */
   xp[1].x = WIDTH-1;   	xp[1].y =  0;
   xp[2].x = (WIDTH-1)/2; 	xp[2].y =  HEIGHT;

   	/* We need to initialize two 1-bit deep WIDTH*HEIGHT pixmaps.
	 *
	 * black_map gets entirely filled with '1's.
	 * tri_map   holds the triangle image.
	 */ 

   black_map = XCreatePixmap(disp,win,WIDTH,HEIGHT,1);
   tri_map   = XCreatePixmap(disp,win,WIDTH,HEIGHT,1);
   gcvalues.foreground = 1;  /* create a gc for the pixmaps */
   gcvalues.background = 0;
   gcmask = GCForeground | GCBackground;
   monogc = XCreateGC(disp,tri_map,gcmask,&gcvalues);

   XFillRectangle(disp,black_map,monogc,0,0,WIDTH,HEIGHT);  /* init black_map */

   XSetForeground(disp,monogc,0);
   XFillRectangle(disp,tri_map,monogc,0,0,WIDTH,HEIGHT);  /* clear tri_map */
   XSetForeground(disp,monogc,1);   
   XFillPolygon(disp,tri_map,monogc,xp,3,Convex,CoordModeOrigin);/* write triangle */

	/* First we copy the tri_map to the window without using a
	 * clipmask. This one comes out OK, producing the upper triangle
	 * in the window.
	 */

   XSetForeground(disp,gc,black);
   XSetBackground(disp,gc,white);
   XCopyPlane(disp,tri_map,win,gc,0,0,WIDTH,HEIGHT,20,20,1); /* this works out */

	/* The second triangle gets created using tri_mask as a clipmask.
	 * A look at the lower triangle in the window shows that this comes
	 * out wrong ---
	 *
	 * The tri_map (with the triangle) gets installed as a clipmask.
	 * Then, black_map (which is set to all '1') gets XCopyPlane()'d
         * through the triangle.
	 * This should reproduce the same triangle image as above; however,
	 * it comes out with some 'bite-marks' (missing pixels) at the right
	 * edge. Calling the program several times can produce missing pixels
	 * at different locations.
	 */

   XSetClipOrigin(disp,gc,20,120);
   XSetClipMask(disp,gc,tri_map);
   XCopyPlane(disp,black_map,win,gc,0,0,WIDTH,HEIGHT,20,120,1); /* this goes wrong */

	/* wait for the user to click the mouse within our window, then quit */ 

   XMaskEvent(disp,ButtonPressMask,&event);
}
   




